@@ -26,7 +26,8 @@
   ;; fixups on the :X86 architecture, NAME is always NIL, so this
   ;; fixup doesn't refer to an external label, and OFFSET is an offset
   ;; from the beginning of the current code block.
-  (offset 0 :read-only t :type (or sb!vm:signed-word #!+x86 label)))
+  (offset 0 :type (or sb!vm:signed-word #!+x86 (or label (cons label fixnum)))
+            :read-only t))
 
 (defstruct (fixup-note
              (:constructor make-fixup-note (kind fixup position))
