@@ -12,21 +12,11 @@
 (in-package "SB!VM")
 
 
-;; If chopping X to 32 bits and sign-extending is equal to the original X,
-;; return the chopped X, which the CPU will always treat as signed.
-;; Notably this allows MOST-POSITIVE-WORD to be an immediate constant.
-(defun immediate32-p (x)
-  (typecase x
-    ((signed-byte 32) x)
-    ((integer #.(- (expt 2 64) (expt 2 31)) #.most-positive-word)
-     (sb!c::mask-signed-field 32 x))
-    (t nil)))
-
-;; If 'immediate32-p' is true, use it; otherwise use a RIP-relative constant.
+;; If 'plausible-signed-imm32-operand-p' is true, use it; otherwise use a RIP-relative constant.
 ;; I couldn't think of a more accurate name for this other than maybe
 ;; 'signed-immediate32-or-rip-relativize' which is just too awful.
 (defun constantize (x)
-  (or (immediate32-p x)
+  (or (plausible-signed-imm32-operand-p x)
       (register-inline-constant :qword x)))
 
 ;;;; unary operations
@@ -288,7 +278,7 @@
   (:generator 2
     (cond ((and (sc-is x any-reg) (sc-is y any-reg) (sc-is r any-reg)
                 (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y :scale 1)))
+           (inst lea r (make-ea :qword :base x :index y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -328,7 +318,7 @@
   (:generator 5
     (cond ((and (sc-is x signed-reg) (sc-is y signed-reg) (sc-is r signed-reg)
                 (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y :scale 1)))
+           (inst lea r (make-ea :qword :base x :index y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -407,7 +397,7 @@
   (:generator 5
     (cond ((and (sc-is x unsigned-reg) (sc-is y unsigned-reg)
                 (sc-is r unsigned-reg) (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y :scale 1)))
+           (inst lea r (make-ea :qword :base x :index y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -1225,7 +1215,7 @@ constant shift greater than word length")))
                     (sc-is y immediate))
            (setf y (tn-value y)))
          (when (integerp y)
-           (acond ((immediate32-p y)
+           (acond ((plausible-signed-imm32-operand-p y)
                    (return-from ensure-not-mem+mem (values x it)))
                   ((typep y '(unsigned-byte 32))
                    ;; Rather than a RIP-relative constant, load a dword (w/o sign-extend)
@@ -1996,7 +1986,7 @@ constant shift greater than word length")))
   (:info mask)
   (:result-types unsigned-num)
   (:generator 4
-     (cond ((or (immediate32-p mask)
+     (cond ((or (plausible-signed-imm32-operand-p mask)
                 (location= x r))
             (loadw r x bignum-digits-offset other-pointer-lowtag)
             (unless (or (eql mask -1)
@@ -2098,7 +2088,7 @@ constant shift greater than word length")))
         DONE
         (unless (or fixnum-mask-p
                     (= mask most-positive-word))
-          (inst and r (or (immediate32-p mask)
+          (inst and r (or (plausible-signed-imm32-operand-p mask)
                           (constantize mask))))))))
 
 (in-package "SB!C")
