@@ -192,10 +192,8 @@
   (:policy :fast-safe)
   (:generator 1
     (move res ptr)
-    ;; Mask the lowtag, and shift the whole address into a positive
-    ;; fixnum.
-    (inst and res (lognot lowtag-mask))
-    (inst shr res 1)))
+    (inst and res (constantize (dpb -1 (byte (- n-word-bits n-fixnum-tag-bits 1)
+                                             n-fixnum-tag-bits) 0)))))
 
 ;;;; allocation
 
@@ -269,7 +267,8 @@
   ;; there is an info-vector in the slot, it has at least one element.
   ;; This would compile to almost the same code without a VOP,
   ;; but using a jmp around a mov instead.
-  (inst lea temp (make-ea :dword :base result :disp (- list-pointer-lowtag)))
+  (inst lea (reg-in-size temp :dword)
+        (make-ea :dword :base result :disp (- list-pointer-lowtag)))
   (inst test (reg-in-size temp :byte) lowtag-mask)
   (inst cmov :e result
         (make-ea-for-object-slot result cons-cdr-slot list-pointer-lowtag)))
@@ -307,13 +306,6 @@
   (:generator 1
     (inst break pending-interrupt-trap)))
 
-#!+sb-thread
-;; 29 signed bits is the max before shifting left by 3 that fits in the
-;; 'displacement' of an EA. This is hugely generous.
-;; We need negative indices to reference the interrupt contexts
-(defknown current-thread-offset-sap ((signed-byte 29))
-  system-area-pointer (flushable))
-
 #!+sb-thread
 (progn
 (define-vop (current-thread-offset-sap/c)
@@ -324,7 +316,7 @@
   (:arg-types (:constant signed-byte))
   (:policy :fast-safe)
   (:generator 1
-    (inst mov sap (thread-tls-ea (ash n 3)))))
+    (inst mov sap (thread-slot-ea n))))
 (define-vop (current-thread-offset-sap)
   (:results (sap :scs (sap-reg)))
   (:result-types system-area-pointer)
